﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;
using BeyondBasicUnitTesting.Business;

namespace BeyondBasicUnitTesting.UnitTests
{
    /// <summary>
    /// Summary description for PersonStubFixture
    /// </summary>
    [TestClass]
    public class PersonStubFixture : UnitTestBase
    {
        public PersonStubFixture()
        {
        }

        [TestMethod]
        public void NothinButMocksBaby()
        {
            MockRepository repository = new MockRepository();

            IPerson person = repository.Stub<IPerson>();
            IPersonFacade facade = repository.Stub<IPersonFacade>();

            facade.IsPersonValid(person);
        }

        [TestMethod]
        public void MockPersonRealFacade_IsValid()
        {
            MockRepository repository = new MockRepository();

            IPerson person = repository.Stub<IPerson>();
            IPersonFacade facade = new PersonFacade();

            Assert.IsFalse(facade.IsPersonValid(null), "Null person should not be valid.");
            Assert.IsFalse(facade.IsPersonValid(person), "Person with all null fields should not be valid.");

            CallIsValidAndAssert(person, facade, "Ben", "Day", "benday@benday.com", "ACTIVE", true);
            CallIsValidAndAssert(person, facade, null, "Day", "benday@benday.com", "ACTIVE", false);
            CallIsValidAndAssert(person, facade, "Ben", null, "benday@benday.com", "ACTIVE", false);
            CallIsValidAndAssert(person, facade, "Ben", "Day", null, "ACTIVE", false);
            CallIsValidAndAssert(person, facade, "Ben", "Day", "benday@benday.com", null, false);
        }

        private void CallIsValidAndAssert(IPerson person, 
            IPersonFacade facade, 
            string firstName, 
            string lastName, 
            string email, 
            string status, 
            bool expectedIsValidResult)
        {
            person.FirstName = firstName;
            person.LastName = lastName;
            person.EmailAddress = email;
            person.Status = status;

            if (expectedIsValidResult == true)
            {
                Assert.IsTrue(facade.IsPersonValid(person), "Person should be valid.");
            }
            else
            {
                Assert.IsFalse(facade.IsPersonValid(person), "Person should not be valid.");
            }
        }
        
        
    }
}
