using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BeyondBasicUnitTesting.Business
{
    public class PersonPresenter : IPersonPresenter
    {
        public PersonPresenter(ILogger logger, IRepository<IPerson> repository)
        {
            if (logger == null)
            {
                throw new ArgumentNullException("logger", "logger is null.");
            }
            if (repository == null)
            {
                throw new ArgumentNullException("repository", "repository is null.");
            }

            m_facadeInstance = new PersonFacade(logger, repository);
        }

        private IPersonFacade m_facadeInstance;
        private IPersonFacade FacadeInstance
        {
            get
            {
                return m_facadeInstance;
            }
        }

        public void InitializeBlank(IPersonSearchView view)
        {
            if (view == null)
            {
                throw new ArgumentNullException("view", "Argument cannot be null.");
            }

            view.Results = null;
        }

        public void ViewAll(IPersonSearchView view)
        {
            if (view == null)
            {
                throw new ArgumentNullException("view", "Argument cannot be null.");
            }

            view.Results = FacadeInstance.GetAll();
        }

        public void LoadById(IPersonDetailView editor)
        {
            if (editor == null)
            {
                throw new ArgumentNullException("editor", "Argument cannot be null.");
            }

            IPerson person = FacadeInstance.GetById(editor.PersonId);

            if (person == null)
            {
                throw new InvalidOperationException("Unknown person id.");
            }

            editor.PersonId = person.Id;
            editor.FirstName = person.FirstName;
            editor.LastName = person.LastName;
            editor.EmailAddress = person.EmailAddress;
            editor.Status = person.Status;
        }

        public void InitializeBlank(IPersonDetailView editor)
        {
            if (editor == null)
            {
                throw new ArgumentNullException("editor", "Argument cannot be null.");
            }

            editor.PersonId = 0;
            editor.FirstName = String.Empty;
            editor.LastName = String.Empty;
            editor.EmailAddress = String.Empty;
            editor.Status = "ACTIVE";
        }

        public void Save(IPersonDetailView view)
        {
            if (view == null)
            {
                throw new ArgumentNullException("editor", "Argument cannot be null.");
            }

            IPerson person;

            if (view.PersonId == 0)
            {
                person = FacadeInstance.Create();
            }
            else
            {
                person = FacadeInstance.GetById(view.PersonId);

                if (FacadeInstance == null)
                {
                    throw new InvalidOperationException("Unknown person id.");
                }
            }

            person.FirstName = view.FirstName;
            person.LastName = view.LastName;
            person.EmailAddress = view.EmailAddress;
            person.Status = view.Status;

            FacadeInstance.SavePerson(person);

            view.PersonId = person.Id;
        }
    }
}
