using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;
using BeyondBasicUnitTesting.Business;
using System.IO;

namespace BeyondBasicUnitTesting.UnitTests
{
    [TestClass]
    public class RepositoryFixture : UnitTestBase
    {
        private static void PopulatePerson(IPerson person, string firstName, string lastName, string email, string status)
        {
            person.FirstName = firstName;
            person.LastName = lastName;
            person.EmailAddress = email;
            person.Status = status;
        }

        [TestMethod]
        public void SavePersonWithRepositorySaveValidPerson()
        {
            MockRepository mocks = new MockRepository();

            IPerson person = mocks.Stub<IPerson>();
            ILogger logger = mocks.DynamicMock<ILogger>();
            IPersonFacade facade = new PersonFacade(logger, new RepositoryMock<IPerson>());

            PopulatePerson(person, "Ben", "Day", "benday@benday.com", "ACTIVE");

            Assert.IsTrue(facade.IsPersonValid(person), "Person should be valid.");

            Assert.AreEqual<int>(0, person.Id, "Id should be zero before save.");

            facade.SavePerson(person);

            Assert.AreNotEqual<int>(0, person.Id, "Person id was still zero after save.");
        }

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void SavePersonWithRepositoryThrowsOnInvalidPerson()
        {
            MockRepository mocks = new MockRepository();

            IPerson person = mocks.Stub<IPerson>();
            ILogger logger = mocks.DynamicMock<ILogger>();
            IPersonFacade facade = new PersonFacade(logger, new RepositoryMock<IPerson>());

            PopulatePerson(person, null, "Day", "benday@benday.com", "ACTIVE");

            Assert.IsFalse(facade.IsPersonValid(person), "Person should be invalid.");

            Assert.AreEqual<int>(0, person.Id, "Id should be zero before save.");

            facade.SavePerson(person);

            Assert.AreNotEqual<int>(0, person.Id, "Person id was still zero after save.");
        }
    }
}
