﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using BeyondBasicUnitTesting.Business;

namespace BeyondBasicUnitTesting.WinUI.Before
{
    public partial class PersonDetail : Form
    {
        public PersonDetail()
        {
            InitializeComponent();
        }

        private void m_btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                IPerson person;

                IPersonFacade facade = new PersonFacade(
                    DemoSession.GetCurrent().LoggerInstance,
                    DemoSession.GetCurrent().PersonRepositoryInstance);

                if (m_labelId.Text == "-1")
                {
                    // create new
                    person = new Person();
                }
                else
                {
                    person = facade.GetById(UiUtility.ParseInt32(m_labelId.Text, -1));

                    if (person == null)
                    {
                        throw new InvalidOperationException("Invalid id.");
                    }
                }

                person.EmailAddress = m_textEmailAddress.Text;
                person.FirstName = m_textFirstName.Text;
                person.LastName = m_textLastName.Text;

                if (m_chkIsActive.Checked == true)
                {
                    person.Status = "ACTIVE";
                }
                else
                {
                    person.Status = "INACTIVE";
                }

                facade.SavePerson(person);

                m_labelId.Text = person.Id.ToString();

                m_status.Text = "Saved.";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            } 
        }

        private void m_btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private int m_personId;

        public void Initialize(int personId)
        {
            m_personId = personId;
        }

        private void LoadById(int id)
        {
            if (id == -1)
            {
                m_labelId.Text = id.ToString();
                m_textEmailAddress.Text = "";
                m_textFirstName.Text = "";
                m_textLastName.Text = "";
                m_chkIsActive.Checked = true;
            }
            else
            {
                IPersonFacade facade = new PersonFacade(
                        DemoSession.GetCurrent().LoggerInstance,
                        DemoSession.GetCurrent().PersonRepositoryInstance);

                IPerson person = facade.GetById(id);

                m_labelId.Text = person.Id.ToString();
                m_textEmailAddress.Text = person.EmailAddress;
                m_textFirstName.Text = person.FirstName;
                m_textLastName.Text = person.LastName;

                if (person.Status == "ACTIVE")
                {
                    m_chkIsActive.Checked = true;
                }
                else
                {
                    m_chkIsActive.Checked = false;
                }
            }
        }

        private void PersonDetail_Load(object sender, EventArgs e)
        {
            LoadById(m_personId);
        }
    }
}
