using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using BeyondBasicUnitTesting.Business;

namespace BeyondBasicUnitTesting.WinUI
{
    public static class UiUtility   
    {
        public static string GetValue(TextBox control)
        {
            return SafeGetValue(control);
        }

        public static string GetValue(Label control)
        {
            return SafeGetValue(control);
        }

        public static DateTime GetValueAsDateTime(TextBox control)
        {
            return GetValueAsDateTime(SafeGetValue(control), DateTime.Now);
        }

        public static DateTime GetValueAsDateTime(Label control)
        {
            return GetValueAsDateTime(SafeGetValue(control), DateTime.Now);
        }

        private static DateTime GetValueAsDateTime(string value, DateTime defaultValue)
        {
            DateTime returnValue;

            if (DateTime.TryParse(value, out returnValue) == true)
            {
                return returnValue;
            }
            else
            {
                return defaultValue;
            }
        }

        public static int GetValueAsInt32(TextBox control)
        {
            return GetValueAsInt32(SafeGetValue(control), 0);
        }

        public static int GetValueAsInt32(Label control)
        {
            return GetValueAsInt32(SafeGetValue(control), 0);
        }

        public static void SetValue(TextBox control, int value)
        {
            SetValue(control, value.ToString());
        }

        public static string SafeGetValue(Label control)
        {
            if (control != null)
            {
                return control.Text;
            }
            else
            {
                return String.Empty;
            }
        }

        public static string SafeGetValue(TextBox control)
        {
            if (control != null)
            {
                return control.Text;
            }
            else
            {
                return String.Empty;
            }
        }

        public static void SetValue(TextBox control, string value)
        {
            control.Text = SafeToString(value);
        }

        public static void SetValue(Label control, string value)
        {
            control.Text = SafeToString(value);
        }

        public static void SetValue(Label control, DateTime value)
        {
            control.Text = value.ToString();
        }

        public static void SetValue(Label control, DateTime value, bool dateOnly)
        {
            if (dateOnly == true)
            {
                control.Text = value.ToString("MM/dd/yyyy");
            }
            else
            {
                SetValue(control, value);
            }
        }

        public static void SetValue(Label control, float value)
        {
            control.Text = value.ToString("C").Replace(",", "");
        }

        public static void SetValue(Label control, int value)
        {
            control.Text = value.ToString();
        }

        public static void SetValue(TextBox control, float value)
        {
            control.Text = value.ToString("C").Replace(",", "");
        }

        public static void SetValue(TextBox control, DateTime value)
        {
            control.Text = value.ToString();
        }

        public static void SetValue(TextBox control, DateTime value, bool dateOnly)
        {
            if (dateOnly == true)
            {
                control.Text = value.ToString("MM/dd/yyyy");
            }
            else
            {
                SetValue(control, value);
            }
        }

        public static int ParseInt32(string value, int defaultReturnValue)
        {
            int returnValue;

            if (Int32.TryParse(value, out returnValue) == true)
            {
                return returnValue;
            }
            else
            {
                return defaultReturnValue;
            }
        }

        public static int GetValueAsInt32(string value, int defaultReturnValue)
        {
            if (value == null)
            {
                return defaultReturnValue;
            }
            else if (value == "-1")
            {
                return -1;
            }
            else
            {
                int returnValue = 0;

                if (Int32.TryParse(value, out returnValue) == true)
                {
                    return returnValue;
                }
                else
                {
                    return defaultReturnValue;
                }
            }
        }

        public static float GetValueAsFloat(Label control)
        {
            return GetValueAsFloat(control.Text, 0f);
        }

        public static float GetValueAsFloat(TextBox control)
        {
            return GetValueAsFloat(control.Text, 0f);
        }

        public static float GetValueAsFloat(string value, float defaultReturnValue)
        {
            if (value == null)
            {
                return defaultReturnValue;
            }
            else if (value == "-1")
            {
                return -1;
            }
            else
            {
                float returnValue = 0;

                if (Single.TryParse(value, out returnValue) == true)
                {
                    return returnValue;
                }
                else
                {
                    if (Single.TryParse(value, System.Globalization.NumberStyles.Currency,
                        null,
                        out returnValue) == true)
                    {
                        return returnValue;
                    }
                    else
                    {
                        return defaultReturnValue;
                    }
                }
            }
        }

        public static string SafeToString(string value, string defaultReturnValue)
        {
            if (value == null)
            {
                return defaultReturnValue;
            }
            else
            {
                return value;
            }
        }

        public static string SafeToString(string value)
        {
            return SafeToString(value, String.Empty);
        }

        public static string SafeToString(float value)
        {
            return SafeToString(value.ToString(), String.Empty);
        }

        /// <summary>
        /// Sets the selection if you are using a control for true and a control for false (for example, two RadioButtons).
        /// </summary>
        /// <param name="trueControl"></param>
        /// <param name="falseControl"></param>
        /// <param name="value"></param>
        public static void SetBooleanOnRadioButtonControls(CheckBox trueControl, CheckBox falseControl, bool value)
        {
            if (value == false)
            {
                trueControl.Checked = false;
                falseControl.Checked = true;
            }
            else
            {
                trueControl.Checked = true;
                falseControl.Checked = false;
            }
        }

        /// <summary>
        /// Gets the selection if you are using a control for true and a control for false (for example, two RadioButtons).
        /// </summary>
        /// <param name="trueControl"></param>
        /// <param name="falseControl"></param>
        public static bool GetBooleanFromRadioButtonControls(CheckBox trueControl, CheckBox falseControl)
        {
            if (trueControl.Checked == true)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public static string GetLastDayOfTheMonth()
        {
            DateTime monthFromToday = DateTime.Today.AddMonths(1);

            DateTime firstOfNextMonth = new DateTime(monthFromToday.Year, monthFromToday.Month, 1);

            DateTime lastDayOfThisMonth = firstOfNextMonth.AddDays(-1);

            return lastDayOfThisMonth.ToShortDateString();
        }

        public static bool HasBaseType(Type typeToCheck, Type baseType)
        {
            bool retValue = false;

            if (typeToCheck != null && baseType != null)
            {
                if (typeToCheck.BaseType != null)
                {
                    if (typeToCheck.BaseType.Equals(baseType))
                        retValue = true;
                    else
                        retValue = HasBaseType(typeToCheck.BaseType, baseType);
                }
            }

            return retValue;
        }
    }
}
